/*
 * This file is part of the SOAFaces Project (hereinafter "SOAFaces").
 * Copyright (C) 2006 Grand Logic, Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package org.soafaces.bundle.client.rpc;

import org.soafaces.services.client.rpc.UniversalClient;
import com.google.gwt.i18n.client.Dictionary;
import com.google.gwt.user.client.rpc.AsyncCallback;

/**
 * Context associated with the configuration of the Weblet.
 *
 * @author Sam Taha
 */
 
public interface WebletContext {
  
  /**
   * Unique ID of the application or workflow/job that this Weblet is part of.
   * Every application or job is assigned a unique id within the container
   * it is running in.
   * 
   * @return
   */
  long getID();
  
  /**
   * Name of the application or workflow/job that this Weblet is part of.
   * 
   * @return
   */
  String getName();
  
  /**
   * Unique ID of Weblet assoicated with continaer this SFB/Weblet is configured in.
   * If this Weblet is associated with a Tasklet then this is also the ID assigned
   * to the Tasklet.
   * @return
   */
  long getBundleID();
  
  /**
   * User defined name associated with Weblet. If this Weblet is associated with
   * a Tasklet then this is also the name assigned to the Tasklet.
   * @return
   */
  String getBundleName();
  
  /**
   * Username of user accessing this Weblet.
   * 
   * @return  Empty string if not defined.
   */
  String getUsername();
  
  /**
   * Any parameters passed to the servlet/URL that references this Weblet can be
   * accessed from this Dictionary.
   * @return
   */
  Dictionary getURLParameters();
  
  /**
   * A Weblet can optionally define a JavaBean in it's MANIFEST.MF file that will then
   * be avilable to the Weblet for reading and writing to. The user defined
   * POJO must implement <code>IsSerializable</code> to be returned as a POJO. If not a
   * POJO that implements IsSerializable it must be fetched as JSON object.
   * <p>
   * This method saves the input JavaBean, that has been fetched via <code>fetchInputBean()</code>, back
   * to the server. The method <code>Weblet.onSaveInputBean()</code>
   * will first be called automatically before the input JavaBean is saved to the server. 
   * </p>
   * 
   * <p>
   * Before trying to save the input JavaBean to the server, the
   * the <code>Weblet.onSaveInputBean()</code>
   * method is called by the container to validate the input JavaBean. Note that
   * implementing/overriding <code>onsaveInputBean()</code> is optional. 
   * If the <code>onSaveInptuBean()</code> call is not successful (that would mean
   * <code>SuccessFailCallback.returnSuccess()</code> is not called) then the 
   * input JavaBean will not be saved to the server and nothing will be
   * returned to this <code>callback</code>.
   * </p>
   * 
   * <ul>
   *  The posssible <code>AsyncCallback.onFailure()</code> exceptions that can be returned
   *  if there is a problem include:
   *  <li><code>BeanTransferException</code> where there is a server-side problem
   *   saving the input JavaBean.</li>
   *  <li>Standard RPC exceptions that can be thrown by GWT.</li>
   * </ul>
   * 
   * @param callback Callback will be invoked when save operation has completed.
   */
  void saveInputBean(AsyncCallback callback);
  
  /**
   * Get the input JavaBean associated with this Weblet from the server and return as
   * a POJO.
   * 
   * <ul>
   *  The posssible <code>FetchPOJOCallback.onFailure()</code> exceptions that can be returned
   *  if there is a problem include:
   *  <li><code>BeanTransferException</code> when there is a server-side problem
   *    fetching the input JavaBean.</li>
   *  <li>Standard RPC exceptions that can be thrown by GWT.</li>
   * </ul>
   * 
   * @param callback Return results of input JavaBean as a POJO.
   */
  void fetchInputBean(FetchPOJOCallback callback);
  
  /**
   * Get the input JavaBean associated with this Weblet from the server and return as
   * a JSON object. Using this method is useful when the object being fetched from the
   * server does not implement <code>IsSerializable</code>.
   * 
   * <ul>
   *  The posssible <code>FetchJSONCallback.onFailure()</code> exceptions that can be returned
   *  if there is a problem include:
   *  <li><code>BeanTransferException</code> where there is a server-side problem
   *   fetching the input JavaBean.</li>
   *  <li>Standard RPC exceptions that can be thrown by GWT.</li>
   * </ul>
   * 
   * @param callback Return results of input JavaBean as a JSON.
   */
  void fetchInputBean(FetchJSONCallback callback);

  /**
   * Get the APP scope level cookies associated with the Tasklet. Cookies are a simple
   * map of name/value properties associated with a Tasklet that can also be viewed by
   * a Tasklet's Weblet GUI.
   * This cookie is available to the
   * Weblet during design mode and also available to the Tasklet running in a job during
   * job processing. APP_SCOPE cookies are shared between all job runs and are essentially global
   * values available to job runs of a Tasklet.
   * 
   * @param callback This callback will be able to return the current cookie map for this Weblet/Tasklet.
   */
  void fetchAppCookie(FetchAppCookieCallback callback);

  /**
   * Save all changes made by this client to the AppCookie. This will apply all set operations
   * back to the server.
   * 
   * @param callback This callback will be able to return the newly updated
   *                 cookie map for this Weblet/Tasklet.
   */
  void saveAppCookie(FetchAppCookieCallback callback);
  
  /**
   * Messaging client that can be used to access Mule endpoint services and SOAFaces
   * endpoint services.
   * @return
   */
  UniversalClient getUniversalClient();
}
