/*
 * This file is part of the SOAFaces Project (hereinafter "SOAFaces").
 * Copyright (C) 2006 Grand Logic, Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package org.soafaces.services.client.rpc;

import com.google.gwt.user.client.rpc.AsyncCallback;
import java.io.Serializable;
import java.util.Map;

/**
 * <p>
 * This is the client interface to the SOAFaces remote messaging API. This interface, for example, can
 * proxy calls through to a remote MuleClient
 * when Mule endpoints are used and can also invoke remote POJO server-side services using the
 * <code>soafaces://</code> protocol identifier. These POJO server-side services are defined within your
 * SOAFaces Bundle (SFB). This allows your Weblet client code to access and invoke
 * remote services from within your GWT client code.
 * </p>
 * <p>
 * SOAFaces services endpoints are identified using the following identifier format:
 * <ul>
 *   <li><code>soafaces://&lt;class name&gt;/&lt;endpoint name&gt;</code></li>
 *   <li>Example: <code>soafaces://com.acme.MyClass/myMethod</code></li>
 * </ul>
 * </p>
 *
 * <p>
 * This interface allows for simple access to a variety of messaging endpoints. It allows
 * GWT clients to make direct calls to Mule endpoints and SOAFaces endpoints and pass POJO
 * and JSON objects and return POJO and JSON objects back from the remote services.
 * </p>
 * <p>
 *  The posssible <code>callback.onFailure()</code> exceptions that can be returned
 *  if there is a problem include:
 * <ul>
 *  <li><code>UniveralClientRemoteException</code> when there is a server-side problem
 *    processing the request.</li>
 *  <li>Standard RPC exceptions that can be thrown by GWT.</li>
 * </ul>
 * </p>
 * 
 * @author Sam Taha
 */
public interface UniversalClient {
  
  /**
   * Call to Mule endpoint or SOAFaces endpoint with payload. Returns an Object that can be of any type
   * supported by GWT serialization.
   * 
   * <p>
   * Payload type can be any built in java type supported by GWT RPC serialization. User defined
   * types that implement IsSerializable are also supported. User defined types that implement
   * java.io.Serializable are NOT supported, built java types that implement java.io.Serializable
   * like String, Integer ...etc are supported. Basic Collections like Map, Lists, and Sets can also
   * be passed and returned.
   * </p>
   * 
   * <p>
   *  In addition, <code>JSONObject</code> is supported. This supports sending a
   * JSON string (as a <code>JSONObject</code>) to the server where it
   * will be (attempted with xstream) serialized into a correspding java object.
   * JSONObjects can be used to map to JavaBean objects on the server. For
   * example if you have the object com.foo.MyBean then to create a 
   * JSONObject to pass as payload you would create it like this:
   * </p>
   *
   * <pre>
   *                 JSONObject props = new JSONObject();
   *                 props.put("name", new JSONString("Joe"));
   *                 props.put("age", new JSONNumber(29));
   *
   *                 JSONObject payload = new JSONObject();
   *                 payload.put("com.foo.MyBean", props);
   *
   *                 universalclient.send("vm://doit", payload, null, callback);
   * </pre>
   * 
   * And like the classical MuleClient, you can using <code>Object[]</code> to pass in
   * multiple arguement. This will work for Mule endpoints and SOAFaces endpoints. The arguments
   * can be mixture of user defined objects (that implement IsSerializable) or JSONObject or
   * Java basic types (like String ...etc). So with this the payload can contain one or multiple
   * arguments to be passed to the service endpoint. Just about all combinations are supported.
   * 
   * @param endpointURI Any standard Mule endpoint or SOAFaces endpoint.
   * @param payload All built in java types supported by GWT serialization can be passed
   *        in as an argument. See details above for more types supported. Multiple arguments
   *        can be passed by sending a java.lang.Object array. This allows you to call services
   *        with multiple arguments. You can mix any objects you want into the Object[], including
   *        JSON objects (that will get marshalled to the remote services as POJOs - very cool!).
   * @param properties Optional and only applicable to Mule/ESB endpoints. Name/Value pair 
   *                   String properties. Map must only contain String
   *                   objects for name and value. Set to null if no properties defined.
   * @param callback Asynchronous callback returning result of remote call. Return types have
   *        same serialization restrictions as payload. You can also return JSON objects. See
   *        examples for details.
   */
  void send(String endpointURI, Object payload, Map<String, Serializable> properties, UniversalClientPOJOCallback callback);
  
  /**
   * Similar to previous call but without Map properties arguement.
   */
  void send(String endpointURI, Object payload, UniversalClientPOJOCallback callback);
  
  /**
   * Similar to previous call and offers and convenient way to call an endpoint 
   * that takes no arguments.
   */
  void send(String endpointURI, UniversalClientPOJOCallback callback);

  /**
   * Similar to previous send methods, except will using JSON to serialize any response
   * returned from the services endpoint.
   */
  void send(String endpointURI, Object payload, Map<String, Serializable> properties, UniversalClientJSONCallback callback);
  
  void send(String endpointURI, Object payload, UniversalClientJSONCallback callback);
  
  void send(String endpointURI, UniversalClientJSONCallback callback);

  /**
   * Asynchrnous version of send methods. This is really only applicable to endpoints 
   * that can asynchronously execute the services like Mule endpoints for example.
   * 
   * @param endpointURI
   * @param payload
   * @param properties
   * @param callback
   */
  void dispatch(String endpointURI, Object payload, Map<String, Serializable> properties, AsyncCallback callback);
  
  void dispatch(String endpointURI, Object payload, AsyncCallback callback);
  
  void dispatch(String endpointURI, AsyncCallback callback);
  
}
